@@ -1,13 +1,14 @@
-use sqlx::{Connect, Executor, PgConnection};
+use sqlx::Postgres;
+use sqlx_test::new;
 
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_query() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let account = sqlx::query!(
         "SELECT * from (VALUES (1, 'Herp Derpinson')) accounts(id, name) where id = $1",
-        1i32,
+        1i32
     )
     .fetch_one(&mut conn)
     .await?;
@@ -20,7 +21,7 @@ async fn test_query() -> anyhow::Result<()> {
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_no_result() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let _ = sqlx::query!("DELETE FROM pg_enum")
         .execute(&mut conn)
@@ -29,10 +30,42 @@ async fn test_no_result() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[cfg_attr(feature = "runtime-async-std", async_std::test)]
+#[cfg_attr(feature = "runtime-tokio", tokio::test)]
+async fn test_text_var_char_char_n() -> anyhow::Result<()> {
+    let mut conn = new::<Postgres>().await?;
+
+    // TEXT
+    // we cannot infer nullability from an expression
+    let rec = sqlx::query!("SELECT 'Hello'::text as greeting")
+        .fetch_one(&mut conn)
+        .await?;
+
+    assert_eq!(rec.greeting.as_deref(), Some("Hello"));
+
+    // VARCHAR(N)
+
+    let rec = sqlx::query!("SELECT 'Hello'::varchar(5) as greeting")
+        .fetch_one(&mut conn)
+        .await?;
+
+    assert_eq!(rec.greeting.as_deref(), Some("Hello"));
+
+    // CHAR(N)
+
+    let rec = sqlx::query!("SELECT 'Hello'::char(5) as greeting")
+        .fetch_one(&mut conn)
+        .await?;
+
+    assert_eq!(rec.greeting.as_deref(), Some("Hello"));
+
+    Ok(())
+}
+
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn _file() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let account = sqlx::query_file!("tests/test-query.sql",)
         .fetch_one(&mut conn)
@@ -52,11 +85,13 @@ struct Account {
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_query_as() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
+    let name: Option<&str> = None;
     let account = sqlx::query_as!(
         Account,
-        "SELECT * from (VALUES (1, null)) accounts(id, name)",
+        "SELECT * from (VALUES (1, $1)) accounts(id, name)",
+        name
     )
     .fetch_one(&mut conn)
     .await?;
@@ -77,7 +112,7 @@ struct RawAccount {
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_query_as_raw() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let account = sqlx::query_as!(
         RawAccount,
@@ -97,7 +132,7 @@ async fn test_query_as_raw() -> anyhow::Result<()> {
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_query_file_as() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let account = sqlx::query_file_as!(Account, "tests/test-query.sql",)
         .fetch_one(&mut conn)
@@ -111,20 +146,26 @@ async fn test_query_file_as() -> anyhow::Result<()> {
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn query_by_string() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let string = "Hello, world!".to_string();
+    let ref tuple = ("Hello, world!".to_string(),);
 
     let result = sqlx::query!(
         "SELECT * from (VALUES('Hello, world!')) strings(string)\
-         where string = $1 or string = $2",
-        string,
-        string[..]
+         where string in ($1, $2, $3, $4, $5, $6, $7)",
+        string, // make sure we don't actually take ownership here
+        &string[..],
+        Some(&string),
+        Some(&string[..]),
+        Option::<String>::None,
+        string.clone(),
+        tuple.0 // make sure we're not trying to move out of a field expression
     )
     .fetch_one(&mut conn)
     .await?;
 
-    assert_eq!(result.string, string);
+    assert_eq!(result.string, Some(string));
 
     Ok(())
 }
@@ -138,7 +179,7 @@ async fn test_nullable_err() -> anyhow::Result<()> {
         name: String,
     }
 
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     let err = sqlx::query_as!(
         Account,
@@ -148,17 +189,19 @@ async fn test_nullable_err() -> anyhow::Result<()> {
     .await
     .unwrap_err();
 
-    if let sqlx::Error::Decode(sqlx::decode::DecodeError::UnexpectedNull) = err {
-        Ok(())
-    } else {
-        panic!("expected `UnexpectedNull`, got {}", err)
+    if let sqlx::Error::Decode(err) = &err {
+        if let Some(sqlx::error::UnexpectedNullError) = err.downcast_ref() {
+            return Ok(());
+        }
     }
+
+    panic!("expected `UnexpectedNullError`, got {}", err)
 }
 
 #[cfg_attr(feature = "runtime-async-std", async_std::test)]
 #[cfg_attr(feature = "runtime-tokio", tokio::test)]
 async fn test_many_args() -> anyhow::Result<()> {
-    let mut conn = connect().await?;
+    let mut conn = new::<Postgres>().await?;
 
     // previous implementation would only have supported 10 bind parameters
     // (this is really gross to test in MySQL)
@@ -170,15 +213,34 @@ async fn test_many_args() -> anyhow::Result<()> {
         .await?;
 
     for (i, row) in rows.iter().enumerate() {
-        assert_eq!(i as i32, row.id);
+        assert_eq!(Some(i as i32), row.id);
     }
 
     Ok(())
 }
 
-async fn connect() -> anyhow::Result<PgConnection> {
-    let _ = dotenv::dotenv();
-    let _ = env_logger::try_init();
+#[cfg_attr(feature = "runtime-async-std", async_std::test)]
+#[cfg_attr(feature = "runtime-tokio", tokio::test)]
+async fn test_array_from_slice() -> anyhow::Result<()> {
+    let mut conn = new::<Postgres>().await?;
 
-    Ok(PgConnection::connect(dotenv::var("DATABASE_URL")?).await?)
+    let list: &[i32] = &[1, 2, 3, 4i32];
+
+    let result = sqlx::query!("SELECT $1::int[] as my_array", list)
+        .fetch_one(&mut conn)
+        .await?;
+
+    assert_eq!(result.my_array, Some(vec![1, 2, 3, 4]));
+
+    println!("result ID: {:?}", result.my_array);
+
+    let account = sqlx::query!("SELECT ARRAY[4,3,2,1] as my_array")
+        .fetch_one(&mut conn)
+        .await?;
+
+    assert_eq!(account.my_array, Some(vec![4, 3, 2, 1]));
+
+    println!("account ID: {:?}", account.my_array);
+
+    Ok(())
 }
